package com.hikcreate.utils

import java.io._
import java.net.{HttpURLConnection, URL}
import java.text.SimpleDateFormat
import java.util
import java.util._

import com.alibaba.fastjson.{JSON, JSONArray, JSONObject}
import com.hikcreate.config.SparkConfig
import org.apache.spark.sql.{DataFrame, SparkSession}

import scala.collection.JavaConversions._
import scalaj.http.Http

/**
  * 通用工具类
  */
object CommonUtils {


  /**
    * 获取sparkSession
    *
    * @return
    */
  def getSparkSession(): SparkSession = {
    val session = SparkSession
      .builder()
      //      .master(SparkConfig.getMaster())
      //      .appName(SparkConfig.getAppName())
      .enableHiveSupport()
      .config("hive.exec.dynamic.partition", true) // 支持 Hive 动态分区
      .config("hive.exec.dynamic.partition.mode", "nonstrict") // 非严格模式
      .config("hive.metastore.uris", "thrift://hadoop01:9083")
      .config("spark.sql.sources.partitionOverwriteMode", "dynamic")
      .config("spark.sql.crossJoin.enabled", "true")
      .getOrCreate()

    session
  }

  /**
    * 获取指定配置文件中的属性
    *
    * @return
    */
  def getProperties(filename: String): Properties = {
    val properties = new Properties()

    properties.load(this.getClass.getClassLoader.getResourceAsStream(filename))

    /*// 指定配置文件路径
    //    val path = Thread.currentThread().getContextClassLoader.getResource(filename).getPath
    val path = "/home/taskConf/" + filename
    // 加载配置文件内容
    properties.load(new FileInputStream(path))*/

    // 返回
    properties
  }


  /**
    * 创建UUID
    *
    * @return
    */
  def createUUID(): String = {
    UUID.randomUUID().toString.replace("-", "").toUpperCase
  }

  /**
    * 保存数据到hive
    *
    * @param dataFrame
    * @param mode
    * @param tableName
    */
  def saveDataToHive(dataFrame: DataFrame, mode: String, partitionday: String, tableName: String): Unit = {
    if (null != partitionday) {
      dataFrame.repartition(1)
        .write
        .mode(mode)
        .format("hive")
        .partitionBy(partitionday)
        .saveAsTable(tableName)
    } else {
      dataFrame
        .write
        .mode(mode)
        .format("hive")
        .saveAsTable(tableName)
    }
  }

  def insertDateIntoTable(dataFrame: DataFrame, mode: String, tableName: String): Unit = {
    dataFrame.repartition(1)
      .write
      .mode(mode)
      .format("hive")
      .insertInto(tableName)
  }

  //计算是否2个日期在30天内
  def yearToYearTheDays(year2: Date): Boolean = {
    //val format2 = new SimpleDateFormat("yyyy-MM-dd")
    val betweenTimes = new Date().getTime - year2.getTime
    val days = (betweenTimes / 1000 / 3600 / 24) - 1
    if (days >= 0 && days <= 30)
      true
    else
      false
  }

  /**
    * 从ods层写数据到dwd层
    *
    * @param ods_table
    * @param dwd_table
    * @param partitions
    * @return
    */
  def odstodwdCommon(ods_table: String, dwd_table: String, partitions: String, partitionday: String): String = {
    val sql =
      s"""
         |INSERT INTO TABLE ${dwd_table} SELECT * from ${ods_table} where ${partitionday}='${partitions}'
      """.stripMargin
    return sql
  }

  /**
    * 获取指定日期前几天的年月日
    *
    * @param dateString 时间字符串
    * @param beforeDays 几天前
    * @return
    */
  def getBeforeDate(dateString: String, beforeDays: Int, format: String): String = {
    // 将时间字符串转换为时间类型
    val date = new SimpleDateFormat(getStrFormat(dateString)).parse(dateString)
    // 获取指定时间是一年中的第一天
    val calendar = Calendar.getInstance()
    calendar.setTime(date)
    val dayOfYear = calendar.get(Calendar.DAY_OF_YEAR)

    // 获取指定日期几天前的日期
    calendar.set(Calendar.DAY_OF_YEAR, dayOfYear - beforeDays)
    val preDay = new SimpleDateFormat(format).format(calendar.getTime)
    preDay
  }

  /**
    * 获取输入的时间字符串的格式
    *
    * @param dateString 时间字符串
    * @return 时间字符串对应的格式
    */
  private def getStrFormat(dateString: String): String = {
    // 要返回的时间格式
    var format = ""
    // 判断时间字符串格式
    if (dateString.contains("-") && dateString.length == 10) {
      format = "yyyy-MM-dd"
    } else if (dateString.contains("/") && dateString.length == 10) {
      format = "yyyy/MM/dd"
    } else if (dateString.length == 8) {
      format = "yyyyMMdd"
    }
    format
  }


  /**
    * 角度换算成弧度
    *
    * @param d 角度
    * @return
    */
  def rad(d: Double): Double = {
    return d * Math.PI / 180.0
  }

  /**
    * 根据经纬度计算两点间的球面距离
    *
    * @param lat1 纬度1
    * @param lng1 经度1
    * @param lat2 纬度2
    * @param lng2 经度2
    * @return
    */
  def getDistance(lat1: Double, lng1: Double, lat2: Double, lng2: Double): Double = {
    var s = 0D
    if (null != lat1 && null != lng1 && null != lat2 && null != lng2) {
      val EARTH_RADIUS = 6371.830D
      val radLat1 = rad(lat1)
      val radLat2 = rad(lat2)
      val radLng1 = rad(lng1)
      val radLng2 = rad(lng2)
      //var s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) + Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)))
      s = Math.acos(Math.cos(radLat1) * Math.cos(radLat2) * Math.cos(radLng1 - radLng2) + Math.sin(radLat1) * Math.sin(radLat2))
      s = s * EARTH_RADIUS
      s = Math.round(s * 1000D) / 1000D
      s
    } else {
      0D
    }
  }

  /**
    * 计算里程数 车辆行驶里程和行驶时间
    *
    * @param len_list
    * @return
    */
  def getAddressLen(len_list: List[String]): List[Seq[String]] = {
    var leg = 0D
    var arr: util.ArrayList[Long] = new util.ArrayList[Long]()
    val daytime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
    var li = new util.ArrayList[String]()
    for (x <- 0 to len_list.length - 2) {
      val lonanlat = len_list.get(x).split(";")
      val lonandlat_next = len_list.get(x + 1).split(";")
      if ("null" != lonanlat(3) && "null" != lonandlat_next(1)) {
        if ("null" != lonanlat(1)) {
          li.add(len_list.get(x))
        } else {
          li.add(lonanlat(0) + ";" + lonandlat_next(1) + ";" + lonandlat_next(2) + ";" + lonanlat(3) + ";" + lonanlat(4))
        }
      } else if ("null" != lonanlat(1) && "null" != lonandlat_next(3)) {
        if ("null" != lonanlat(3)) {
          li.add(len_list.get(x))
        } else {
          li.add(lonandlat_next(0) + ";" + lonanlat(1) + ";" + lonanlat(2) + ";" + lonandlat_next(3) + ";" + lonandlat_next(4))
        }
      }
    }
    getDriverLength(li)
  }

  /**
    * 计算行驶里程
    *
    * @param arr
    * @return
    */
  def getDriverLength(arr: util.ArrayList[String]): List[Seq[String]] = {
    val arrs = arr.sortBy(x => x.substring(0, 19))
    var leg = 0D
    var arlis = new util.ArrayList[Long]()
    var arr_li = new util.ArrayList[Tuple4[String, String, Double, Double]]()
    val daytime = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
    for (x <- 0 to arrs.length - 2) {
      val str = arrs(x).split(";")
      val str1 = arrs(x + 1).split(",")
      val time1 = daytime.parse(str(0)).getTime
      val time2 = daytime.parse(str1(0)).getTime
      val len = getDistance(str(4).toDouble, str(3).toDouble, str1(4).toDouble, str1(3).toDouble)
      if (len != 0 && time1 != time2 && str(2) == str1(2)) {
        val times = time2 - time1
        arr_li.add((str(1), str(2), times, len))
      }
    }
    val tupleToTuple = arr_li.groupBy(x => (x._1, x._2)).mapValues(x => (x.map(x => x._3).reduce(_ + _), x.map(x => x._4).reduce(_ + _)))
    val tuples = tupleToTuple.map(x => Seq(x._1._1, x._1._2, x._2._1.toString, x._2._2.toString))
    tuples.toList
  }

  /**
    * 根据经纬度获取地理位置
    *
    * @param lon
    * @param lat
    * @return
    */
  def interfaceUrl(lon: String, lat: String): String = {
    try {
      val path = "http://10.197.236.100:40612/bcpbase/geocode/regeo"
      val url = new URL(path)
      val conn: HttpURLConnection = url.openConnection().asInstanceOf[HttpURLConnection]
      var out: PrintWriter = null
      //设置通用的请求属性
      conn.setRequestProperty("accept", "*/*")
      // 连接复用
      conn.setRequestProperty("connection", "Keep-Alive")
      conn.setRequestProperty("user-agent", "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; SV1)")
      //设置是否向httpUrlConnection输出，设置是否从httpUrlConnection读入，此外发送post请求必须设置这两个
      //最常用的Http请求无非是get和post，get请求可以获取静态页面，也可以把参数放在URL字串后面，传递给servlet，
      //post与get的 不同之处在于post的参数不是放在URL字串里面，而是放在http请求的正文内。
      conn.setDoOutput(true)
      conn.setDoInput(true)
      //获取URLConnection对象对应的输出流
      out = new PrintWriter(conn.getOutputStream())
      //发送请求参数即数据
      out.print(lat)
      //缓冲数据
      out.flush()
      //获取URLConnection对象对应的输入流
      val is = conn.getInputStream()
      //构造一个字符流缓存
      val br = new BufferedReader(new InputStreamReader(is, "GBK"))
      var str = ""
      var flage = true
      while (flage) {
        val s = br.readLine()
        if (s == null) {
          flage = false
        } else {
          str += s
        }
      }
      //关闭流
      is.close()
      conn.disconnect()
      str
    } catch {
      case e: Exception => e.printStackTrace()
        null
    }
  }

  import java.text.SimpleDateFormat

  /**
    * 两个时间差
    *
    * @param a
    * @param b
    * @return 秒数
    */
  def getBetweenSecond(a: String, b: String): Long = {
    val format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss")
    var startDate: Date = null
    var endDate: Date = null
    var ss = 0L
    startDate = format.parse(a)
    endDate = format.parse(b)
    try {
      ss = endDate.getTime - startDate.getTime
    } catch {
      case e: Exception =>
        e.printStackTrace()
    }
    ss / 1000
  }

  def getAddressAndLocationCode(lon: Double, lat: Double): (String, String) = {
    val json = new JSONObject()
    val arr = new JSONArray()
    val lonAndLat = new JSONObject()
    lonAndLat.put("longitude", lon)
    lonAndLat.put("latitude", lat)
    arr.add(lonAndLat)
    json.put("locations", arr)
    val response = Http("http://10.197.236.100:40612/bcpbase/geocode/regeo")
      .postData(json.toJSONString)
      .header("content-type", "application/json")
      //.charset("ISO-8859-1")
      .timeout(connTimeoutMs = 8000, readTimeoutMs = 8000)
      .asString
    if (response.code == 200) {
      val body = JSON.parseObject(response.body)
      val item = body.getJSONObject("result").getJSONArray("regeoItems").getJSONObject(0)
      val address = item.getString("formattedAddress")
      val locationCode = item.getJSONObject("addressComponent").getString("adcode")
      (address, locationCode)
    } else {
      throw new RuntimeException("http请求城市区编码出错")
    }
  }

  //根据经纬度获取地区编码
  def getLocationCode(lon: Double, lat: Double): (String, String, String) = {
    val locationCode = getAddressAndLocationCode(lon, lat)._2
    val provinceCode = locationCode.substring(0, 2)
    val cityCode = locationCode.substring(2, 4)
    val areaCode = locationCode.substring(4, 6)
    (provinceCode, cityCode, areaCode)
  }

  def isIDNumber(IDNumber: String): Boolean = {
    if (IDNumber == null || "" == IDNumber) return false
    // 定义判别用户身份证号的正则表达式（15位或者18位，最后一位可以为字母）
    val regularExpression = "(^[1-9]\\d{5}(18|19|20)\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}[0-9Xx]$)|" + "(^[1-9]\\d{5}\\d{2}((0[1-9])|(10|11|12))(([0-2][1-9])|10|20|30|31)\\d{3}$)"
    val matches = IDNumber.matches(regularExpression)
    //判断第18位校验值
    if (matches) if (IDNumber.length == 18) try {
      val charArray = IDNumber.toCharArray
      //前十七位加权因子
      val idCardWi = Array(7, 9, 10, 5, 8, 4, 2, 1, 6, 3, 7, 9, 10, 5, 8, 4, 2)
      //这是除以11后，可能产生的11位余数对应的验证码
      val idCardY = Array("1", "0", "X", "9", "8", "7", "6", "5", "4", "3", "2")
      var sum = 0
      var i = 0
      while ( {
        i < idCardWi.length
      }) {
        val current = String.valueOf(charArray(i)).toInt
        val count = current * idCardWi(i)
        sum += count

        {
          i += 1;
          i - 1
        }
      }
      val idCardLast = charArray(17)
      val idCardMod = sum % 11
      if (idCardY(idCardMod).toUpperCase == String.valueOf(idCardLast).toUpperCase) return true
      else {
        return false
      }
    } catch {
      case e: Exception =>
        e.printStackTrace()
        return false
    }
    matches
  }
}
